\version "2.24.1"

#(begin

; StaffGroup contexts
 (define groups '(Flutes Oboes Bassoons Horns Violins Violas))

; Staff contexts
 (define instruments '(Flute FluteI FluteII Oboe OboeI OboeII Bassoon BassoonI BassoonII
                       Horn HornI HornII HornIII HornIV Trumpet
                       Timpani
                       Violin ViolinI ViolinII ViolinIII ViolinIV
                       Viola ViolaI ViolaII
                       Cello Bass))

; Pair context (Staff)
 (define pairs '(FluteIandII OboeIandII CorAnglaisIandII ClarinetIandII BassoonIandII
                 TrumpetIandII HornIandII HornIIIandIV
                 ViolaIandII CB CBB))

 (define notenames   #("C" "D" "E" "F" "G" "A" "B"))
 (define alterations #("♭" "" "♯"))

 (define (add-context-prop symbol type?)
   (set-object-property! symbol 'translation-type? type?)
   (set-object-property! symbol 'translation-doc "custom context property")
   (set! all-translation-properties (cons symbol all-translation-properties)) symbol)

 (add-context-prop 'key     ly:pitch?)
 (add-context-prop 'tuning  ly:music?)
 (add-context-prop 'barlist list?)
 (add-context-prop 'channel ly:dispatcher?)

 (define-markup-command (oneinst layout props n) (markup?)
   #:properties ((width 3))
  "Instrument X"
  (let* ((staffspace (ly:output-def-lookup layout 'staff-space))            ; staff-space in mm
         (indent     (/ (ly:output-def-lookup layout 'indent) staffspace))) ; indent in staff-space units
    (interpret-markup layout props
      #{
        \markup
          \override #`(line-width . ,indent)
          \fill-line \vcenter { \null \hcenter-in #width #n }
      #}
    )))

 (define-markup-command (pairinsts layout props X Y i) (markup? markup? markup?)
   #:properties ((width 3))
  "Instrument X and Y"
  (let* ((staffspace (ly:output-def-lookup layout 'staff-space))            ; staff-space in mm
         (indent     (/ (ly:output-def-lookup layout 'indent) staffspace))) ; indent in staff-space units
    (interpret-markup layout props
      #{
        \markup
          \override #`(line-width . ,indent)
          \fill-line \vcenter { \null #i \column { \hcenter-in #width #X \hcenter-in #width #Y } }
      #}
    )))
)

% Engraver to transpose music automatically
Auto_transpose_engraver =
#(lambda(context)
  ; transpose engraved music
  (let* ((transp (ly:context-property context 'key (ly:context-property context 'instrumentTransposition))))
    (define (auto-transp engraver music) (ly:music-transpose music (ly:pitch-negate transp)))

    ; create engraver
    (make-engraver
      (listeners
        ; transpose note-event
        ((note-event engraver event)
          (auto-transp engraver (ly:event-property event 'music-cause)))
        ; transpose key-signature
        ((key-change-event engraver event)
          (auto-transp engraver (ly:event-property event 'music-cause)))))
  )
)

Transposing_instrument_name =
#(lambda(context)
  ; set instrumentName to instrumentName in key
  (let* ((cxtname   (symbol->string (ly:context-name context)))
         (name      (ly:context-property context 'instrumentName))
         (short     (ly:context-property context 'shortInstrumentName))
         (transp    (ly:context-property context
                      'key
                      (ly:context-property context 'instrumentTransposition
                      #{ c' #} )))
         (key       (vector-ref notenames (ly:pitch-notename transp)))
         (alter     (vector-ref alterations (1+ (* 2 (ly:pitch-alteration transp)))))
         (withkey   (string-append name " in " key alter))
         (octave    (cond
                      ((equal? transp #{ c'   #} ) "(alto)" )
                      ((equal? transp #{ bes  #} ) "(alto)" )
                      ((equal? transp #{ a    #} ) "(alto)" )
                      ((equal? transp #{ c    #} ) "(basso)")
                      ((equal? transp #{ bes, #} ) "(basso)")
                      ((equal? transp #{ a,   #} ) "(basso)")
                      (else #f)))
         (fullname  (if (and octave (string-contains cxtname "Horn"))
                      (markup #:center-column (withkey octave))
                      (markup withkey)))
         (parent    (ly:context-parent context))
         (pname     (ly:context-property parent 'instrumentName #f))
         (pshort    (ly:context-property parent 'shortInstrumentName #f)))

    (if pname (set! fullname (markup #:null)))
    (ly:context-set-property! context 'instrumentName
        (cond ((string-contains cxtname "IandII")   (markup (#:pairinsts "I"   "II" fullname)))
              ((string-contains cxtname "IIIandIV") (markup (#:pairinsts "III" "IV" fullname)))
              (else (markup fullname))))

    (ly:context-set-property! context 'shortInstrumentName (if pshort (markup #:null) short)))

  (make-engraver)
)

instrumentTuning =
#(lambda(context)
  (let* ((chord (ly:context-property context 'tuning))
         (ev-chord (car (extract-named-music chord 'EventChord)))
         (pitches  (event-chord-pitches ev-chord))
         (tuning   (map (lambda(p) (vector-ref notenames (ly:pitch-notename p))) pitches))
         (alter    (map (lambda(p) (vector-ref alterations (1+ (* 2 (ly:pitch-alteration p))))) pitches))
         (notes    (map (lambda(a b) (string-append a b)) tuning alter))
         (name     (ly:context-property context 'instrumentName)))

      (ly:context-set-property! context 'instrumentName
        (string-append name " in " (string-join notes ", "))))

  (make-engraver)
)

autoTranspose = \with {
  \remove "Key_engraver"
  \consists \Auto_transpose_engraver
  \consists "Key_engraver"
}

groupcxt =
#(define-scheme-function (group) (symbol?)
  #{
    \context {
      \StaffGroup
      \name #(symbol->string group)
      \alias StaffGroup
      \accepts Staff
      \consists "Keep_alive_together_engraver"
      systemStartDelimiter = #'SystemStartBrace % staves with same instrument connected by brace
      \override SystemStartBrace.collapse-height = 5 % supress brace when only one staff active
    }
  #}
)

paircxt =
#(define-scheme-function (group) (symbol?)
  #{
    \context {
      \Staff
      \name #(symbol->string group)
      #(if final #{ \RemoveEmptyStaves #})
      \alias Staff
      \accepts Voice
      \consists "Merge_rests_engraver"
      \override VerticalAxisGroup.remove-layer = 2
      aDueText   = \markup \whiteout "a2"
      soloText   = \markup \whiteout "Solo"
      soloIIText = \markup \whiteout "SoloII"
    }
  #}
)

staffcxt =
#(define-scheme-function (instrument) (symbol?)
  #{
    \context {
      \Staff
      \name #(symbol->string instrument)
      #(if final #{ \RemoveEmptyStaves #})
      \alias Staff
      \accepts Voice
      \override VerticalAxisGroup.remove-layer = 1
    }
  #}
)

mergestaff =
#(define-scheme-function () ()
  (make-apply-context
    (lambda (context)
      (let* ((pairparent  (not (string-null? (ly:context-id context))))
             (staffparent (ly:context-find context 'Staff))
             (parentname  (ly:context-name staffparent)))
        (if (not pairparent)
          (ly:context-set-property! staffparent 'keepAliveInterfaces '()))))))

separatestaff = \unset Staff.keepAliveInterfaces

setAccepts =
#(define-void-function (ctx-name contexts layout) (symbol? symbol-list? ly:output-def?)
  (let* ((module  (ly:output-def-scope layout))
         (ctx-def (module-ref module ctx-name))
         (modlist (map (lambda(i) `(accepts ,i)) contexts))
         (ctx-mod (ly:make-context-mod modlist)))
    (ly:output-def-set-variable! layout ctx-name (ly:context-def-modify ctx-def ctx-mod))))

hierarchy =
#(define-void-function (output) (ly:output-def?)
  (setAccepts 'Score      '(Timpani)                          output)
  (setAccepts 'StaffGroup (append groups pairs instruments)   output)
  (setAccepts 'StaffGroup '(ViolinISolo ViolinIISolo)         output)
  (setAccepts 'StaffGroup '(ViolinSolo ViolaSolo)             output)
  (setAccepts 'StaffGroup '(CelloSolo BassSolo)               output)
  (setAccepts 'Violins    '(ViolinI   ViolinII)               output)
  (setAccepts 'Violins    '(ViolinIII ViolinIV)               output)
  (setAccepts 'Violas     '(ViolaI ViolaII ViolaIandII)       output)
  (setAccepts 'Flutes     '(FluteI FluteII FluteIandII)       output)
  (setAccepts 'Oboes      '(OboeI OboeII OboeIandII)          output)
  (setAccepts 'Bassoons   '(BassoonI BassoonII BassoonIandII) output)
  (setAccepts 'Horns      '(HornIandII HornIIIandIV)          output)
  (setAccepts 'Horns      '(HornI HornII HornIII HornIV)      output)
  (setAccepts 'Horns      '(Horn)                             output)
)
